package com.example.testproject;

import java.util.ArrayList;
import java.util.List;

import com.vaadin.data.Container;
import com.vaadin.data.Item;
import com.vaadin.data.util.IndexedContainer;

public class ChannelController {

	/**
	 * Interface used for listening to channel list selection changes.
	 * 
	 * @author pyy
	 * 
	 */
	public interface ChannelSelectionChangeListener {
		public void channelEntryChanged(String id);
	}

	/**
	 * Interface used for listening to playlist selection changes.
	 * @author pyy
	 *
	 */
	public interface PlaylistSelectionChangeListener {
		public void playlistEntryChanged (String id);
	}
	
	/**
	 * List of available channels.
	 */
	private IndexedContainer channelList;

	/**
	 * List of available videos in channel.
	 */
	private IndexedContainer videoList;

	/**
	 * List of <code>entryChangeListener</code> that are notified when a channel
	 * selection changes.
	 */
	private List<ChannelSelectionChangeListener> entryChangeListeners = new ArrayList<ChannelSelectionChangeListener>();

	/**
	 * List of <code>PlaylistChangeListener</code> that are notified when a playlist
	 * selection changes.
	 */
	private List<PlaylistSelectionChangeListener> selectionChangeListeners = new ArrayList<PlaylistSelectionChangeListener>();
	
	/**
	 * Constructor.
	 * 
	 * Create initial containers for channel list and playlist and initialize
	 * them; channel list to a available channels for us and playlist to empty.
	 */
	public ChannelController() {
		channelList = new IndexedContainer();
		channelList.addContainerProperty("Music Channel", String.class, null);

		videoList = new IndexedContainer();
		videoList.addContainerProperty("Artist", String.class, null);
		videoList.addContainerProperty("Title", String.class, null);
		videoList.addContainerProperty("Duration", String.class, null);

		refreshChannelList();
		resetPlaylist();
	}

	/**
	 * Return available channels.
	 * 
	 * @return
	 */
	public Container getChannelList() {
		return channelList;
	}

	/**
	 * Refreshes the available channel list.
	 */
	public void refreshChannelList() {
		// Test data
		Item item = channelList.addItem("1");
		item.getItemProperty("Music Channel").setValue("Hollywood Records");

		item = channelList.addItem("2");
		item.getItemProperty("Music Channel").setValue("Michael Jackson's Channel");

		item = channelList.addItem("3");
		item.getItemProperty("Music Channel").setValue("Madonna");

		// TODO: Obtain registered channels that are available from data
		// sources
	}

	/**
	 * Adds listener for channel entry changes.
	 * 
	 * @param listener
	 *            <code>EntryChangeListener</code> interface implementation
	 */
	public void addChannelListener(ChannelSelectionChangeListener listener) {
		entryChangeListeners.add(listener);
	}
	
	/**
	 * Adds listener for playlist entry changes.
	 * 
	 * @param listener
	 *            <code>EntryChangeListener</code> interface implementation
	 */
	public void addPlaylistListener(PlaylistSelectionChangeListener listener) {
		selectionChangeListeners.add(listener);
	}

	/**
	 * Notify all listeners that channel entry <code>id</code> is selected.
	 * 
	 * @param id
	 */
	public void selectChannelEntry(String id) {
		for (ChannelSelectionChangeListener entry : entryChangeListeners) {
			entry.channelEntryChanged(id);
		}
	}

	/**
	 * Notify all listeners that playlist entry <code>id</code> is selected.
	 * 
	 * @param id
	 */
	public void selectPlaylistEntry(String id) {
		for (PlaylistSelectionChangeListener entry : selectionChangeListeners) {
			entry.playlistEntryChanged(id);
		}
	}

	
	/**
	 * Load playlist for <code>channelId</code>.
	 * 
	 * @param channelId
	 */
	public void loadPlaylist(String channelId) {
		//System.out.println("Loading playlist for " + jukeboxId);
		if (null == channelId) {
			return;
		}
		// remove old playlist data
		resetPlaylist();

		// load new playlist data
		// test data
		createPlaylist(channelId);
	}

	/**
	 * Return playlist of currently selected channel.
	 * 
	 * @return
	 */
	public Container getPlaylist() {
		return videoList;
	}

	/**
	 * Resets the playlist.
	 */
	public void resetPlaylist() {
		videoList.removeAllItems();
	}

	/* TESTING */

	/**
	 * Test data creation.
	 * 
	 * @param id
	 */
	private void createPlaylist(String id) {
		Item item;
		// Hollywood Records
		if (id.equals("1")) {
			item = videoList.addItem("HRqOjhNN4hQ");
			item.getItemProperty("Artist").setValue("Vanessa Hudgens");
			item.getItemProperty("Title").setValue("Sneakernight");
			item.getItemProperty("Duration").setValue("3:08");
			
			item = videoList.addItem("zR5_KkxgGqE");
			item.getItemProperty("Artist").setValue("Jesse McCartney");
			item.getItemProperty("Title").setValue("Leavin'");
			item.getItemProperty("Duration").setValue("3:39");
			
			item = videoList.addItem("0z1kSdk7y1A");
			item.getItemProperty("Artist").setValue("Jonas Brothers");
			item.getItemProperty("Title").setValue("Burnin' Up");
			item.getItemProperty("Duration").setValue("3:27");
			
			item = videoList.addItem("L8p5t4uPCZQ");
			item.getItemProperty("Artist").setValue("Aly & AJ");
			item.getItemProperty("Title").setValue("Like Whoa");
			item.getItemProperty("Duration").setValue("2:33");
			
			item = videoList.addItem("m-jRHI3INyg");
			item.getItemProperty("Artist").setValue("Miley Cyrus");
			item.getItemProperty("Title").setValue("Start All Over");
			item.getItemProperty("Duration").setValue("3:59");
		} 
		else if (id.equals("2"))
		{
			item = videoList.addItem("WObfcDIf6lY");
			item.getItemProperty("Artist").setValue("Michael Jackson");
			item.getItemProperty("Title").setValue("Beat It");
			item.getItemProperty("Duration").setValue("5:23");
			
			item = videoList.addItem("4FZcAzZOyOg");
			item.getItemProperty("Artist").setValue("Michael Jackson");
			item.getItemProperty("Title").setValue("Earth Song");
			item.getItemProperty("Duration").setValue("6:09");
			
			item = videoList.addItem("_fHoDWc22B0");
			item.getItemProperty("Artist").setValue("Michael Jackson");
			item.getItemProperty("Title").setValue("Billy Jean");
			item.getItemProperty("Duration").setValue("4:53");
			
			item = videoList.addItem("iDUbAgfrvU0");
			item.getItemProperty("Artist").setValue("Michael Jackson");
			item.getItemProperty("Title").setValue("Smooth Criminal");
			item.getItemProperty("Duration").setValue("0:00");
			
			item = videoList.addItem("33XPuLXJ6p0");
			item.getItemProperty("Artist").setValue("Michael Jackson");
			item.getItemProperty("Title").setValue("Will You Be There");
			item.getItemProperty("Duration").setValue("4:23");
		}
		else if (id.equals("3"))
		{
			item = videoList.addItem("UYZ12yav6GU");
			item.getItemProperty("Artist").setValue("Madonna");
			item.getItemProperty("Title").setValue("Erotic");
			item.getItemProperty("Duration").setValue("0:00");
			
			item = videoList.addItem("OyobCisF9iM");
			item.getItemProperty("Artist").setValue("Madonna");
			item.getItemProperty("Title").setValue("Liquid Love");
			item.getItemProperty("Duration").setValue("4:35");
			
			item = videoList.addItem("BfRZAfYbNCk");
			item.getItemProperty("Artist").setValue("Madonna");
			item.getItemProperty("Title").setValue("You Thrill Me");
			item.getItemProperty("Duration").setValue("2:45");
			
			item = videoList.addItem("lWTDR6ztcnQ");
			item.getItemProperty("Artist").setValue("Madonna");
			item.getItemProperty("Title").setValue("Celebration");
			item.getItemProperty("Duration").setValue("3:49");
			
			item = videoList.addItem("60hYX8ssBm4");
			item.getItemProperty("Artist").setValue("Madonna");
			item.getItemProperty("Title").setValue("Get Stupid");
			item.getItemProperty("Duration").setValue("3:05");
		}		
	}
}
